@@ -22,7 +22,7 @@ from pm4py.algo.discovery.dfg.utils.dfg_utils import get_activities_from_dfg, \
 from pm4py.algo.discovery.dfg.utils.dfg_utils import get_ingoing_edges, get_outgoing_edges
 from pm4py.algo.discovery.dfg.utils.dfg_utils import negate, get_activities_self_loop, transform_dfg_to_directed_nx_graph
 from pm4py.algo.discovery.dfg.variants import native as dfg_inst
-from pm4py.objects.dfg.filtering.dfg_filtering import clean_dfg_based_on_noise_thresh
+from pm4py.algo.filtering.dfg.dfg_filtering import clean_dfg_based_on_noise_thresh
 from pm4py.algo.discovery.inductive.variants.im.util import base_case, fall_through
 from pm4py import util as pmutil
 from pm4py.algo.discovery.inductive.variants.im.util import splitting as split
@@ -30,10 +30,24 @@ from pm4py.algo.discovery.inductive.util import parallel_cut_utils, detection_ut
 from pm4py.statistics.attributes.log import get as attributes_get
 from pm4py.statistics.end_activities.log import get as end_activities_get
 from pm4py.statistics.start_activities.log import get as start_activities_get
-from pm4py.algo.discovery.inductive.parameters import Parameters
 from pm4py.util import exec_utils
 from pm4py.objects.log.util import filtering_utils
 import logging
+from pm4py.util import constants
+from enum import Enum
+
+
+class Parameters(Enum):
+    ACTIVITY_KEY = constants.PARAMETER_CONSTANT_ACTIVITY_KEY
+    START_TIMESTAMP_KEY = constants.PARAMETER_CONSTANT_START_TIMESTAMP_KEY
+    TIMESTAMP_KEY = constants.PARAMETER_CONSTANT_TIMESTAMP_KEY
+    CASE_ID_KEY = constants.PARAMETER_CONSTANT_CASEID_KEY
+    NOISE_THRESHOLD = "noiseThreshold"
+    EMPTY_TRACE_KEY = "empty_trace"
+    ONCE_PER_TRACE_KEY = "once_per_trace"
+    CONCURRENT_KEY = "concurrent"
+    STRICT_TAU_LOOP_KEY = "strict_tau_loop"
+    TAU_LOOP_KEY = "tau_loop"
 
 
 class SubtreePlain(object):
