@@ -24,6 +24,10 @@ from pm4py.util.constants import PARAMETER_CONSTANT_ATTRIBUTE_KEY, PARAMETER_CON
 from pm4py.util.xes_constants import DEFAULT_NAME_KEY, DEFAULT_RESOURCE_KEY, DEFAULT_TIMESTAMP_KEY
 import deprecation
 
+from typing import Optional, Dict, Any, Union, Tuple, List
+from pm4py.objects.log.obj import EventLog, EventStream, Trace
+import itertools
+
 
 class Parameters(Enum):
     ATTRIBUTE_KEY = PARAMETER_CONSTANT_ATTRIBUTE_KEY
@@ -276,7 +280,7 @@ def A_eventually_B_eventually_C_eventually_D(log, A, B, C, D, parameters=None):
     return new_log
 
 
-def eventually_follows(log, attribute_values, parameters=None):
+def eventually_follows(log: EventLog, attribute_values: List[str], parameters: Optional[Dict[Union[str, Parameters], Any]] = None) -> EventLog:
     """
     Applies the eventually follows rule
 
@@ -301,6 +305,8 @@ def eventually_follows(log, attribute_values, parameters=None):
     if parameters is None:
         parameters = {}
 
+    log = log_converter.apply(log, variant=log_converter.Variants.TO_EVENT_LOG, parameters=parameters)
+
     attribute_key = exec_utils.get_param_value(Parameters.ATTRIBUTE_KEY, parameters, DEFAULT_NAME_KEY)
     timestamp_key = exec_utils.get_param_value(Parameters.TIMESTAMP_KEY, parameters, DEFAULT_TIMESTAMP_KEY)
 
@@ -312,48 +318,29 @@ def eventually_follows(log, attribute_values, parameters=None):
                        omni_present=log.omni_present, properties=log.properties)
 
     for trace in log:
-        if enable_timestamp:
-            occurrences = [[trace[i][timestamp_key].timestamp() for i in range(len(trace)) 
-                            if attribute_key in trace[i] and trace[i][attribute_key] == attribute_value] for attribute_value in attribute_values]
-        else:
-            occurrences = [[i for i in range(len(trace)) 
-                            if attribute_key in trace[i] and trace[i][attribute_key] == attribute_value] for attribute_value in attribute_values]
+        occurrences = [[i for i in range(len(trace))
+                        if attribute_key in trace[i] and trace[i][attribute_key] == attribute_value] for attribute_value in attribute_values]
 
+        is_good = False
 
-        is_good = True
-        if enable_timestamp and timestamp_diff_boundaries:
-            prev_min = min(occurrences[0], default=-1)
-            for i in range(1, len(attribute_values)):
-                if prev_min == -1 or len(occurrences[i]) == 0:
-                    is_good = False
+        for c in itertools.product(*occurrences):
+            ok = True
+            for i in range(len(c)-1):
+                if c[i] > c[i+1]:
+                    ok = False
                     break
+            if ok:
+                if enable_timestamp and timestamp_diff_boundaries:
+                    for i in range(len(c)-1):
+                        timest_i = trace[i][timestamp_key].timestamp()
+                        timest_j = trace[i+1][timestamp_key].timestamp()
+                        if timest_j - timest_i < timestamp_diff_boundaries[i][0] or timest_j - timest_i > timestamp_diff_boundaries[i][1]:
+                            ok = False
+                            break
 
-                if timestamp_diff_boundaries:
-                    min_diff = timestamp_diff_boundaries[i - 1][0]
-                    max_diff = timestamp_diff_boundaries[i - 1][1]
-                    min_timestamp = min([o for o in occurrences[i] if (o - prev_min) >= min_diff and (o - prev_min) <= max_diff], default=-1)
-                else:
-                    min_timestamp = min([o for o in occurrences[i] if o >= prev_min], default = -1)
-
-                prev_min = min_timestamp
-
-                if prev_min == -1:
-                    is_good = False
-                    break
-                
-        else:        
-            prev_min = min(occurrences[0], default=-1)
-            for i in range(1, len(attribute_values)):
-                if prev_min == -1:
-                    is_good = False
-                    break
-
-                if len(occurrences[i]) == 0:
-                    is_good = False
-                    break
-
-                min_index = min([o for o in occurrences[i] if o >= prev_min], default = -1)
-                prev_min = min_index
+            if ok:
+                is_good = True
+                break
 
         if is_good:
             if positive:
@@ -364,7 +351,7 @@ def eventually_follows(log, attribute_values, parameters=None):
     return new_log
 
 
-def A_next_B_next_C(log, A, B, C, parameters=None):
+def A_next_B_next_C(log: EventLog, A: str, B: str, C: str, parameters: Optional[Dict[Union[str, Parameters], Any]] = None) -> EventLog:
     """
     Applies the A next B next C rule
 
@@ -392,8 +379,7 @@ def A_next_B_next_C(log, A, B, C, parameters=None):
     if parameters is None:
         parameters = {}
 
-    if not isinstance(log, EventLog):
-        log = log_converter.apply(log, variant=log_converter.TO_EVENT_LOG, parameters=parameters)
+    log = log_converter.apply(log, variant=log_converter.Variants.TO_EVENT_LOG, parameters=parameters)
 
     attribute_key = exec_utils.get_param_value(Parameters.ATTRIBUTE_KEY, parameters, DEFAULT_NAME_KEY)
     positive = exec_utils.get_param_value(Parameters.POSITIVE, parameters, True)
@@ -423,7 +409,7 @@ def A_next_B_next_C(log, A, B, C, parameters=None):
     return new_log
 
 
-def four_eyes_principle(log, A, B, parameters=None):
+def four_eyes_principle(log: EventLog, A: str, B: str, parameters: Optional[Dict[Union[str, Parameters], Any]] = None) -> EventLog:
     """
     Verifies the Four Eyes Principle given A and B
 
@@ -450,8 +436,7 @@ def four_eyes_principle(log, A, B, parameters=None):
     if parameters is None:
         parameters = {}
 
-    if not isinstance(log, EventLog):
-        log = log_converter.apply(log, variant=log_converter.TO_EVENT_LOG, parameters=parameters)
+    log = log_converter.apply(log, variant=log_converter.Variants.TO_EVENT_LOG, parameters=parameters)
 
     attribute_key = exec_utils.get_param_value(Parameters.ATTRIBUTE_KEY, parameters, DEFAULT_NAME_KEY)
     resource_key = exec_utils.get_param_value(Parameters.RESOURCE_KEY, parameters, DEFAULT_RESOURCE_KEY)
@@ -477,7 +462,7 @@ def four_eyes_principle(log, A, B, parameters=None):
     return new_log
 
 
-def attr_value_different_persons(log, A, parameters=None):
+def attr_value_different_persons(log: EventLog, A: str, parameters: Optional[Dict[Union[str, Parameters], Any]] = None) -> EventLog:
     """
     Checks whether an attribute value is assumed on events done by different resources
 
@@ -500,8 +485,7 @@ def attr_value_different_persons(log, A, parameters=None):
     if parameters is None:
         parameters = {}
 
-    if not isinstance(log, EventLog):
-        log = log_converter.apply(log, variant=log_converter.TO_EVENT_LOG, parameters=parameters)
+    log = log_converter.apply(log, variant=log_converter.Variants.TO_EVENT_LOG, parameters=parameters)
 
     attribute_key = exec_utils.get_param_value(Parameters.ATTRIBUTE_KEY, parameters, DEFAULT_NAME_KEY)
     resource_key = exec_utils.get_param_value(Parameters.RESOURCE_KEY, parameters, DEFAULT_RESOURCE_KEY)
