@@ -16,6 +16,11 @@
 '''
 from enum import Enum
 from pm4py.util import constants
+import warnings
+
+
+warnings.warn(
+    "pm4py.visualization.dfg.parameters is deprecated. please use the parameters inside the visualization variant.")
 
 
 class Parameters(Enum):
@@ -26,3 +31,4 @@ class Parameters(Enum):
     END_ACTIVITIES = "end_activities"
     TIMESTAMP_KEY = constants.PARAMETER_CONSTANT_TIMESTAMP_KEY
     START_TIMESTAMP_KEY = constants.PARAMETER_CONSTANT_START_TIMESTAMP_KEY
+    FONT_SIZE = "font_size"
