@@ -20,7 +20,7 @@ from graphviz import Digraph
 
 from pm4py.objects.petri_net.obj import Marking
 from pm4py.objects.petri_net import properties as petri_properties
-from pm4py.util import exec_utils
+from pm4py.util import exec_utils, constants
 from enum import Enum
 from pm4py.util.constants import PARAMETER_CONSTANT_ACTIVITY_KEY, PARAMETER_CONSTANT_TIMESTAMP_KEY, DEFAULT_ARTIFICIAL_START_ACTIVITY, DEFAULT_ARTIFICIAL_END_ACTIVITY
 
@@ -66,7 +66,7 @@ def apply(net, initial_marking, final_marking, decorations=None, parameters=None
     debug = exec_utils.get_param_value(Parameters.DEBUG, parameters, False)
     set_rankdir = exec_utils.get_param_value(Parameters.RANKDIR, parameters, None)
     font_size = exec_utils.get_param_value(Parameters.FONT_SIZE, parameters, "12")
-    bgcolor = exec_utils.get_param_value(Parameters.BGCOLOR, parameters, "transparent")
+    bgcolor = exec_utils.get_param_value(Parameters.BGCOLOR, parameters, constants.DEFAULT_BGCOLOR)
 
     return graphviz_visualization(net, image_format=image_format, initial_marking=initial_marking,
                                   final_marking=final_marking, decorations=decorations, debug=debug,
@@ -74,7 +74,7 @@ def apply(net, initial_marking, final_marking, decorations=None, parameters=None
 
 
 def graphviz_visualization(net, image_format="png", initial_marking=None, final_marking=None, decorations=None,
-                           debug=False, set_rankdir=None, font_size="12", bgcolor="transparent"):
+                           debug=False, set_rankdir=None, font_size="12", bgcolor=constants.DEFAULT_BGCOLOR):
     """
     Provides visualization for the petrinet
 
@@ -174,6 +174,15 @@ def graphviz_visualization(net, image_format="png", initial_marking=None, final_
 
     # add arcs, in order by their source and target objects names, to avoid undeterminism in the visualization
     arcs_sort_list = sorted(list(net.arcs), key=lambda x: (x.source.name, x.target.name))
+
+    # check if there is an arc with weight different than 1.
+    # in that case, all the arcs in the visualization should have the arc weight visible
+    arc_weight_visible = False
+    for arc in arcs_sort_list:
+        if arc.weight != 1:
+            arc_weight_visible = True
+            break
+
     for a in arcs_sort_list:
         arrowhead = "normal"
         if petri_properties.ARCTYPE in a.properties:
@@ -187,7 +196,7 @@ def graphviz_visualization(net, image_format="png", initial_marking=None, final_
         elif a in decorations and "color" in decorations[a]:
             viz.edge(str(id(a.source)), str(id(a.target)), color=decorations[a]["color"], fontsize=font_size, arrowhead=arrowhead)
         else:
-            if a.weight > 1:
+            if arc_weight_visible:
                 viz.edge(str(id(a.source)), str(id(a.target)), fontsize=font_size, arrowhead=arrowhead, label=str(a.weight))
             else:
                 viz.edge(str(id(a.source)), str(id(a.target)), fontsize=font_size, arrowhead=arrowhead)
